;;-----------------------------LICENSE NOTICE-----------------------------------
;;  This file is part of Turtle Slam: An Amstrad CPC Game
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU General Public License for more details.
;;
;;  You should have received a copy of the GNU General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;------------------------------------------------------------------------------

.area _DATA
.area _CODE

.include "models/bullet.h.s"
.include "models/room.h.s"
.include "models/hero.h.s"
.include "game.h.s"
.include "enemy.h.s"
.include "hero.h.s"
.include "drawer.h.s"
.include "video.h.s"
.include "cpctelera.h.s"

.macro ReserveBullets
	.rept 10
		DefineDefaultBullet
	.endm
	.db #255
.endm

bullets::
	ReserveBullets

moveUp:
	ld a, b_y(ix)
	sub a, #4
	ld b_y(ix), a
	jp continue1_controlBullet

moveDown:
	ld a, b_y(ix)
	add a, #4
	ld b_y(ix), a
	jp continue1_controlBullet

moveRight:
	ld a, b_x(ix)
	add a, #1
	ld b_x(ix), a
	jp continue1_controlBullet

moveLeft:
	ld a, b_x(ix)
	sub a, #1
	ld b_x(ix), a
	jp continue1_controlBullet

repeat_bullet:
	.db #3

controlBullet::
	ld ix, #bullets
	loop_controlBullet:
		ld a, 0(ix)
		cp a, #255
		jp z, finish_controlBullet
		ld a, b_st(ix)
		cp a, #2
		jp nz, next_controlBullet
		ld a, b_di(ix)
		sub a, #2
		jp z, moveUp
		dec a
		jp z, moveRight
		dec a
		jp z, moveDown
		dec a
		jp z, moveLeft
	continue1_controlBullet:
		call borderBullet
		ld a, b_si(ix)
		cp a, #0
		jp z, enemyBullet
	heroBullet:
		call collisionBullet
		call enemy_collisionBullet
		jp next_controlBullet
	enemyBullet:
		call hero_collisionBullet
	next_controlBullet:
		ld bc, #b_size
		add ix, bc
		jp loop_controlBullet
	finish_controlBullet:
		ld a, (repeat_bullet)
		dec a
		jp z, repeat_controlBullet
		ld (repeat_bullet), a
		ret
	repeat_controlBullet:
		ld a, #2
		ld (repeat_bullet), a
		jp controlBullet

hero_collisionBullet:
	ld a, (hero_inv)
	cp a, #1
	ret z
	ld b, b_x(ix) ;; X1
	ld d, b_x(ix)
	inc d ;; X2
	ld c, b_y(ix) ;; Y1
	ld a, c
	add a, #4
	ld e, a ;; Y2
	ld iy, #hero
	xhorizontal1:
		ld a, e_x(iy)
		sub a, b
		jp z, xvertical1
		jp m, xhorizontal11
	xhorizontal2:
		ld a, e_x(iy)
		sub a, d
		jp m, xhorizontal22
		ret
	xvertical1:
		ld a, e_y(iy)
		sub a, c
		jp z, xcollided
		jp m, xvertical11
	xvertical2:
		ld a, e_y(iy)
		sub a, e
		jp m, xvertical22
		ret
	xhorizontal11:
		add a, #3
		jp m, xhorizontal2
		jp xvertical1
	xhorizontal22:
		add a, #3
		ret m
		jp xvertical1
	xvertical11:
		add a, #13
		jp m, xvertical2
		jp xcollided
	xvertical22:
		add a, #13
		ret m
	xcollided:
		call disableOneBullet
		push af
		push ix
		push iy
		ld l, #01
		ld h, #15
		ld e, #58
		ld d, #01
		ld bc, #0000
		ld a, #0b00000100
		call cpct_akp_SFXPlay_asm
		pop iy
		pop ix
		pop af	
		ld a, #30
		ld (hero_inv_counter), a
		ld a, #1
		ld (hero_inv), a
		ld a, h_hp(iy)
		dec a
		ld h_hp(iy), a
		ret

enemy_collisionBullet::
	ld iy, #enemies
	enemy_loop2_collisionBullet:
		ld a, 0(iy)
		cp a, #255
		ret z
		ld a, en_hp(iy)
		cp a, #0
		jp z, enemy_next_vc
		enemy_horizontal1::
			ld a, e_x(iy)
			sub a, b_x(ix)
			jp z, enemy_vertical1
			jp m, enemy_horizontal11
		enemy_horizontal2::
			ld a, e_x(iy)
			sub a, b_x(ix)
			dec a
			jp m, enemy_horizontal22
			jp enemy_next_vc
		enemy_vertical1::
			ld a, e_y(iy)
			sub a, b_y(ix)
			jp z, enemy_collided
			jp m, enemy_vertical11
		enemy_vertical2::
			ld a, e_y(iy)
			sub a, b_y(ix)
			sub a, #4
			jp m, enemy_vertical22
		enemy_next_vc::
			ld bc, #en_size
			add iy, bc
			jp enemy_loop2_collisionBullet
		enemy_horizontal11::
			add a, e_w(iy)
			dec a
			jp m, enemy_horizontal2
			jp enemy_vertical1
		enemy_horizontal22::
			add a, e_w(iy)
			dec a
			jp m, enemy_next_vc
			jp enemy_vertical1
		enemy_vertical11::
			add a, e_h(iy)
			dec a
			jp m, enemy_vertical2
			jp enemy_collided
		enemy_vertical22::
			add a, e_h(iy)
			dec a
			jp m, enemy_next_vc
		enemy_collided::
			ld a, en_hp(iy)
			dec a
			ld en_hp(iy), a
			push af
			push ix
			push iy
			ld l, #03
			ld h, #15
			ld e, #58
			ld d, #01
			ld bc, #0000
			ld a, #0b00000100
			call cpct_akp_SFXPlay_asm
			pop iy
			pop ix
			pop af
			jp z, erase_enemy
			jp disableOneBullet
		erase_enemy:
			call disableOneBullet
			push ix
			ld ix, #hero
			call draw
			pop ix
			call cpct_waitVSYNC_asm
       		call switchBuffers
			push ix
			push iy
			pop ix
			call erase
			pop ix
			call cpct_waitVSYNC_asm
       		call switchBuffers
			push ix
			ld ix, #hero
			call erase
			pop ix
			ret

borderBullet:
	ld a, b_x(ix)
	cp a, #rd_lel
	jp z, disableOneBullet
	cp a, #rd_ril
	jp z, disableOneBullet
	ld a, b_y(ix)
	cp a, #rd_upl
	jp z, disableOneBullet
	cp a, #rd_lol
	jp z, disableOneBullet
	ret

collisionBullet:
	call getRoom
	ld l, r_pt_l(iy)
	ld h, r_pt_h(iy)
	push hl
	pop iy
	
	loop2_collisionBullet:
		ld a, 0(iy)
		cp a, #255
		ret z

		horizontal1:
			ld a, e_x(iy)
			sub a, b_x(ix)
			jp z, vertical1
			jp m, horizontal11
		horizontal2:
			ld a, e_x(iy)
			sub a, b_x(ix)
			dec a
			jp m, horizontal22
			jp next_vc
		vertical1:
			ld a, e_y(iy)
			sub a, b_y(ix)
			jp m, vertical11
		vertical2:
			ld a, e_y(iy)
			sub a, b_y(ix)
			sub a, #4
			jp m, vertical22
		next_vc:
			ld bc, #e_size
			add iy, bc
			jp loop2_collisionBullet
		horizontal11:
			add a, e_w(iy)
			dec a
			jp m, horizontal2
			jp vertical1
		horizontal22:
			add a, e_w(iy)
			dec a
			jp m, next_vc
			jp vertical1
		vertical11:
			add a, e_h(iy)
			dec a
			jp m, vertical2
			jp collided
		vertical22:
			add a, e_h(iy)
			dec a
			jp m, next_vc
		collided::
			jp disableOneBullet
	ret

disableOneBullet::
	ld a, #1
	ld b_st(ix), a
	ret

disableBullet::
	ld ix, #bullets
	loop_disableBullet:
		ld a, 0(ix)
		cp a, #255
		ret z
		ld a, #0
		ld b_st(ix), a
		ld a, #40
		ld b_x(ix), a
		ld a, #20
		ld b_y(ix), a
		ld bc, #b_size
		add ix, bc
	jp loop_disableBullet