;;-----------------------------LICENSE NOTICE-----------------------------------
;;  This file is part of Turtle Slam: An Amstrad CPC Game
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU General Public License for more details.
;;
;;  You should have received a copy of the GNU General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;------------------------------------------------------------------------------

.area _DATA
.area _CODE

.include "tilemaps/myfinalset_pack.h.s"
.include "tilemaps/tilemap.h.s"
.include "models/hero.h.s"
.include "models/bullet.h.s"
.include "models/map.h.s"
.include "cpctelera.h.s"
.include "video.h.s"
.include "hero.h.s"
.include "bullet.h.s"
.include "enemy.h.s"
.include "timer.h.s"
.include "game.h.s"

;; Sprites
.globl _sprite_door_ov
.globl _sprite_door_oh
.globl _sprite_door_cv
.globl _sprite_door_ch
.globl _sprite_door_sov
.globl _sprite_door_soh
.globl _sprite_door_scv
.globl _sprite_door_sch
.globl _sprite_obstacle
.globl _sprite_spike
.globl _sprite_heart
.globl _sprite_ChicagosBird

phase:
	.db #0

eraseEnemy::
	ld ix, #enemies
	loop_eraseEnemy:
		ld a, 0(ix)
		cp a, #255
		ret z
		ld a, en_hp(ix)
		cp a, #0
		call nz, erase
		ld bc, #en_size
		add ix, bc
		jp loop_eraseEnemy

drawEnemy::
	ld ix, #enemies
	loop_drawEnemy:
		ld a, 0(ix)
		cp a, #255
		ret z
		ld a, en_hp(ix)
		cp a, #0
		call nz, draw
		ld bc, #en_size
		add ix, bc
		jp loop_drawEnemy

;; ============================================================================
;; Draw entity
;; Input parameters:
;;      IX: Pointer to entity
;; Destroyed register values: AF, BC, DE, HL
;; ============================================================================
draw::	
	;; Get screen pointer
	ld de, (ptrVideo)
	ld c, e_x(ix)
	ld b, e_y(ix)
	call cpct_getScreenPtr_asm
	ex de, hl
	;; Get sprite
	ld h, h_pSpr_h(ix)
	ld l, h_pSpr_l(ix)
	;; Get dimensions
	ld c, e_w(ix)
	ld b, e_h(ix)
	jp cpct_drawSprite_asm

;; ============================================================================
;; Erase old sprite of dinamic entity
;; Input parameters:
;;      IX: Pointer to dinamic entity
;; Destroyed register values: AF, BC, DE, HL
;; ============================================================================
erase::
	;; Get screen pointer
	ld de, (ptrVideo)
	ld c, h_ox(ix)
	ld b, h_oy(ix)
	call cpct_getScreenPtr_asm
	;; Erase
	ex de, hl
	ld a, #0
	ld c, e_w(ix)
	ld b, e_h(ix)
	call cpct_drawSolidBox_asm
	;; Update position
	ld a, e_x(ix)
	ld h_ox(ix), a
	ld a, e_y(ix)
	ld h_oy(ix), a
	ret

decompress_buffer = 0x040
levelmaxsize = 0x314
decompress_buffer_end = decompress_buffer + levelmaxsize - 1
tileset_ptr = 0x040
tilemap_ptr = decompress_buffer + 288

drawRoom::
	push iy
	;; Decompression
	ld  hl, #_myfinalset_pack_end
	ld  de, #decompress_buffer_end
	call cpct_zx7b_decrunch_s_asm
	;; Set tilemap
	ld	bc, #0x1914
	ld	de, #_level_W
	ld	hl, #tileset_ptr
	call cpct_etm_setDrawTilemap4x8_ag_asm
	;; Draw tilemap
	ld hl, (ptrVideo)
	ld	de, #tilemap_ptr
	call cpct_etm_drawTilemap4x8_ag_asm
	;; Draw doors
	pop iy
	ld a, r_v(iy)
	cp a, #1
	jp z, visited
		call drawClosedDoors
		jp continue1
	visited:
		call drawOpenDoors
	continue1::
	;; Draw chicagos 30 bird
	ld bc, #0x0144
	ld de, (ptrVideo)
	call cpct_getScreenPtr_asm
	ex de, hl
	ld bc, #0x0D06
	ld hl, #_sprite_ChicagosBird
	call cpct_drawSprite_asm
	;; Draw obstacles
	ld a, #0
	ld (phase), a
	ld bc, #_sprite_obstacle
	ld (obstacle_sprite), bc

	ld l, r_pt_l(iy)
	ld h, r_pt_h(iy)
	push hl
	pop ix
	loop_draw_obstacles:
		ld a, 0(ix)
		cp a, #255
		jp z, continue2
		;; b -> h // c -> w // d -> y // e -> x
		ld d, e_y(ix)
		ld b, e_h(ix)
	newRowObstacle:
		ld e, e_x(ix)
		ld c, e_w(ix)

	drawObstacle:
		push bc ;; A
		push de ;; B
		push de ;; C
		pop bc ;; C
		ld de, (ptrVideo)
		call cpct_getScreenPtr_asm
		ex de, hl
	obstacle_sprite = . + 1
		ld hl, #_sprite_obstacle
		ld bc, #0x1004
		call cpct_drawSprite_asm
		pop de ;; B
		pop bc ;; A
		ld a, e
		add a, #4
		ld e, a
		ld a, c
		sub a, #4
		ld c, a
		cp a, #0
		jp nz, drawObstacle

	nextRoowObstacle:
		ld a, d
		add a, #16
		ld d, a
		ld a, b
		sub a, #16
		ld b, a
		jp z, next_draw_obstacles
		jp newRowObstacle

	next_draw_obstacles:
		ld bc, #e_size
		add ix, bc
		jp loop_draw_obstacles

	draw_spikes:
		inc ix
		inc a
		ld (phase), a
		ld bc, #_sprite_spike
		ld (obstacle_sprite), bc
		jp loop_draw_obstacles

	continue2:
		ld a, (phase)
		cp a, #0
		jp z, draw_spikes
		;; Draw hp
		ld ix, #hero
		call drawHp
	ret

drawHp::
	ld de, (ptrVideo)
	ld a, #0x03
	ld bc, #0x0B16
	call cpct_drawSolidBox_asm
	ld ix, #hero
	ld a, #1
	ld (heart_pos), a
	ld a, h_hp(ix)
	cp a, #0
	ret z
	loop_drawHp:
		push af
		ld de, (ptrVideo)
		heart_pos = . + 1
		ld bc, #0x0101
		call cpct_getScreenPtr_asm
		ex de, hl
		ld hl, #_sprite_heart
		ld bc, #0x0A04
		call cpct_drawSprite_asm
		ld a, (heart_pos)
		add a, #4
		ld (heart_pos), a
		pop af
		dec a
		jp nz, loop_drawHp
	ret

drawOpenDoors::
	ld a, rd_up(iy)
	cp a, #1
	call z, drawNormalOpenUpperDoor
	ld a, rd_up(iy)
	cp a, #2
	call z, drawSpecialOpenUpperDoor
	ld a, rd_lo(iy)
	cp a, #1
	call z, drawNormalOpenLowerDoor
	ld a, rd_lo(iy)
	cp a, #2
	call z, drawSpecialOpenLowerDoor
	ld a, rd_ri(iy)
	cp a, #1
	call z, drawNormalOpenRightDoor
	ld a, rd_ri(iy)
	cp a, #2
	call z, drawSpecialOpenRightDoor
	ld a, rd_le(iy)
	cp a, #1
	call z, drawNormalOpenLeftDoor
	ld a, rd_le(iy)
	cp a, #2
	call z, drawSpecialOpenLeftDoor
	ret

drawClosedDoors:
	ld a, rd_up(iy)
	cp a, #1
	call z, drawNormalClosedUpperDoor
	ld a, rd_up(iy)
	cp a, #2
	call z, drawSpecialClosedUpperDoor
	ld a, rd_lo(iy)
	cp a, #1
	call z, drawNormalClosedLowerDoor
	ld a, rd_lo(iy)
	cp a, #2
	call z, drawSpecialClosedLowerDoor
	ld a, rd_ri(iy)
	cp a, #1
	call z, drawNormalClosedRightDoor
	ld a, rd_ri(iy)
	cp a, #2
	call z, drawSpecialClosedRightDoor
	ld a, rd_le(iy)
	cp a, #1
	call z, drawNormalClosedLeftDoor
	ld a, rd_le(iy)
	cp a, #2
	call z, drawSpecialClosedLeftDoor
	ret

drawNormalOpenUpperDoor:
	ld de, (ptrVideo)
	ld bc, #0x0825
	call cpct_getScreenPtr_asm
	ex de, hl
	ld bc, #0x0806
	ld hl, #_sprite_door_oh
	jp cpct_drawSprite_asm

drawNormalOpenLowerDoor:
	ld de, (ptrVideo)
	ld bc, #0xC025
	call cpct_getScreenPtr_asm
	ex de, hl
	ld bc, #0x0806
	ld hl, #_sprite_door_oh
	jp cpct_drawSprite_asm

drawNormalOpenRightDoor:
	ld de, (ptrVideo)
	ld bc, #0x604E
	call cpct_getScreenPtr_asm
	ex de, hl
	ld bc, #0x1002
	ld hl, #_sprite_door_ov
	jp cpct_drawSprite_asm

drawNormalOpenLeftDoor:
	ld de, (ptrVideo)
	ld bc, #0x6000
	call cpct_getScreenPtr_asm
	ex de, hl
	ld bc, #0x1002
	ld hl, #_sprite_door_ov
	jp cpct_drawSprite_asm

drawNormalClosedUpperDoor:
	ld de, (ptrVideo)
	ld bc, #0x0825
	call cpct_getScreenPtr_asm
	ex de, hl
	ld bc, #0x0806
	ld hl, #_sprite_door_ch
	jp cpct_drawSprite_asm

drawNormalClosedLowerDoor:
	ld de, (ptrVideo)
	ld bc, #0xC025
	call cpct_getScreenPtr_asm
	ex de, hl
	ld bc, #0x0806
	ld hl, #_sprite_door_ch
	jp cpct_drawSprite_asm

drawNormalClosedRightDoor:
	ld de, (ptrVideo)
	ld bc, #0x604E
	call cpct_getScreenPtr_asm
	ex de, hl
	ld bc, #0x1002
	ld hl, #_sprite_door_cv
	jp cpct_drawSprite_asm

drawNormalClosedLeftDoor:
	ld de, (ptrVideo)
	ld bc, #0x6000
	call cpct_getScreenPtr_asm
	ex de, hl
	ld bc, #0x1002
	ld hl, #_sprite_door_cv
	jp cpct_drawSprite_asm

drawSpecialOpenUpperDoor:
	ld de, (ptrVideo)
	ld bc, #0x0825
	call cpct_getScreenPtr_asm
	ex de, hl
	ld bc, #0x0806
	ld hl, #_sprite_door_soh
	jp cpct_drawSprite_asm

drawSpecialOpenLowerDoor:
	ld de, (ptrVideo)
	ld bc, #0xC025
	call cpct_getScreenPtr_asm
	ex de, hl
	ld bc, #0x0806
	ld hl, #_sprite_door_soh
	jp cpct_drawSprite_asm

drawSpecialOpenRightDoor:
	ld de, (ptrVideo)
	ld bc, #0x604E
	call cpct_getScreenPtr_asm
	ex de, hl
	ld bc, #0x1002
	ld hl, #_sprite_door_sov
	jp cpct_drawSprite_asm

drawSpecialOpenLeftDoor:
	ld de, (ptrVideo)
	ld bc, #0x6000
	call cpct_getScreenPtr_asm
	ex de, hl
	ld bc, #0x1002
	ld hl, #_sprite_door_sov
	jp cpct_drawSprite_asm

drawSpecialClosedUpperDoor:
	ld de, (ptrVideo)
	ld bc, #0x0825
	call cpct_getScreenPtr_asm
	ex de, hl
	ld bc, #0x0806
	ld hl, #_sprite_door_sch
	jp cpct_drawSprite_asm

drawSpecialClosedLowerDoor:
	ld de, (ptrVideo)
	ld bc, #0xC025
	call cpct_getScreenPtr_asm
	ex de, hl
	ld bc, #0x0806
	ld hl, #_sprite_door_sch
	jp cpct_drawSprite_asm

drawSpecialClosedRightDoor:
	ld de, (ptrVideo)
	ld bc, #0x604E
	call cpct_getScreenPtr_asm
	ex de, hl
	ld bc, #0x1002
	ld hl, #_sprite_door_scv
	jp cpct_drawSprite_asm

drawSpecialClosedLeftDoor:
	ld de, (ptrVideo)
	ld bc, #0x6000
	call cpct_getScreenPtr_asm
	ex de, hl
	ld bc, #0x1002
	ld hl, #_sprite_door_scv
	jp cpct_drawSprite_asm

drawBullet::
		ld ix, #bullets	
	loop1_drawBullet:
		ld a, 0(ix)
		cp a, #255
		jp z, continue2_drawBullet
		ld a, b_st(ix)
		cp a, #0
		jp z, continue1_drawBullet
		ld de, (ptrVideo)
		ld c, b_x(ix)
		ld b, b_y(ix)
		call cpct_getScreenPtr_asm
		ex de, hl
		push ix
		pop hl
		ld bc, #b_p1
		add hl, bc
		ex de, hl
		ld bc, #0x0401
		call cpct_getScreenToSprite_asm
	continue1_drawBullet:
		ld bc, #b_size
		add ix, bc
		jp loop1_drawBullet
	continue2_drawBullet:
		ld ix, #bullets
	loop2_drawBullet:
		ld a, 0(ix)
		cp a, #255
		ret z
		ld a, b_st(ix)
		cp a, #2
		jp nz, continue3_drawBullet
		ld de, (ptrVideo)
		ld c, b_x(ix)
		ld b, b_y(ix)
		call cpct_getScreenPtr_asm
		ex de, hl
		ld bc, #0x0401
		ld a, b_si(ix)
		cp a, #0
		jp z, redBullet
	blueBullet:
		ld a, #0xf3
		jp colourBullet
	redBullet:
		ld a, #0xc0
	colourBullet:
		call cpct_drawSolidBox_asm
	continue3_drawBullet:
		ld bc, #b_size
		add ix, bc
		jp loop2_drawBullet

eraseBullet::
	ld ix, #bullets
	loop_eraseBullet:
		ld a, 0(ix)
		cp a, #255
		ret z
		ld a, b_st(ix)
		cp a, #0
		jp z, continue_eraseBullet
		cp a, #2
		jp z, erase_eraseBullet
		ld a, #0
		ld b_st(ix), a
		ld b_p1(ix), a
		ld b_p2(ix), a
		ld b_p3(ix), a
		ld b_p4(ix), a
		ld a, #40
		ld b_x(ix), a
		ld a, #20
		ld b_y(ix), a
	erase_eraseBullet:
		ld de, (ptrVideo)
		ld c, b_ox(ix)
		ld b, b_oy(ix)
		call cpct_getScreenPtr_asm
		ex de, hl
		push ix
		pop hl
		ld bc, #b_op1
		add hl, bc
		ld bc, #0x0401
		call cpct_drawSprite_asm
		ld a, b_x(ix)
		ld b_ox(ix), a
		ld a, b_y(ix)
		ld b_oy(ix), a
		ld a, b_p1(ix)
		ld b_op1(ix), a
		ld a, b_p2(ix)
		ld b_op2(ix), a
		ld a, b_p3(ix)
		ld b_op3(ix), a
		ld a, b_p4(ix)
		ld b_op4(ix), a
	continue_eraseBullet:
		ld bc, #b_size
		add ix, bc
	jp loop_eraseBullet

sec_aux:
	.db #0

min_aux:
	.db #0

drawTime::
	ld a, (refresh_sec)
	dec a
	ret z
	ld (refresh_sec), a
	ld a, (sec)
drawTime_sec1:
	sub a, #10
	jp m, xx0x
	sub a, #10
	jp m, xx1x
	sub a, #10
	jp m, xx2x
	sub a, #10
	jp m, xx3x
	sub a, #10
	jp m, xx4x
	sub a, #10
	jp xx5x
drawTime_sec2:
	ld a, (sec_aux)
	inc a
	jp p, xxx9
	inc a
	jp p, xxx8
	inc a
	jp p, xxx7
	inc a
	jp p, xxx6
	inc a
	jp p, xxx5
	inc a
	jp p, xxx4
	inc a
	jp p, xxx3
	inc a
	jp p, xxx2
	inc a
	jp p, xxx1
	jp xxx0
drawTime_min1:
	ld a, (refresh_min)
	dec a
	ret z

	ld bc, #0x011D
	ld de, (ptrVideo)
	call cpct_getScreenPtr_asm
	ex de, hl
	ld bc, #0x0A01
	ld hl, #_sprite_sep
	call cpct_drawSprite_asm

	ld (refresh_min), a
	ld a, (min)
	sub a, #10
	jp m, y0xxx
	sub a, #10
	jp m, y1xxx
	sub a, #10
	jp m, y2xxx
	sub a, #10
	jp m, y3xxx
	sub a, #10
	jp m, y4xxx
	sub a, #10
	sub a, #10
	jp m, y5xxx
	sub a, #10
	jp m, y6xxx
	sub a, #10
	jp m, y7xxx
	sub a, #10
	jp m, y8xxx
	sub a, #10
	jp y9xxx
drawTime_min2:
	ld a, (min_aux)
	cp a, #0
	jp p, x9xx
	inc a
	jp p, x9xx
	inc a
	jp p, x8xx
	inc a
	jp p, x7xx
	inc a
	jp p, x6xx
	inc a
	jp p, x5xx
	inc a
	jp p, x4xx
	inc a
	jp p, x3xx
	inc a
	jp p, x2xx
	inc a
	jp p, x1xx
	jp x0xx

.globl _sprite_0
.globl _sprite_1
.globl _sprite_2
.globl _sprite_3
.globl _sprite_4
.globl _sprite_5
.globl _sprite_6
.globl _sprite_7
.globl _sprite_8
.globl _sprite_9
.globl _sprite_sep

nnnn1:
	ld de, (ptrVideo)
	call cpct_getScreenPtr_asm
	ex de, hl
	ld bc, #0x0A02
	ret

xxnx:
	ld (sec_aux), a
	ld bc, #0x011F
	call nnnn1
	ret

xxxn:
	ld bc, #0x0122
	call nnnn1
	ret

nxxx:
	ld (min_aux), a
	ld bc, #0x0117
	call nnnn1
	ret

xnxx:
	ld bc, #0x011A
	call nnnn1
	ret

xx0x:
	call xxnx
	ld hl, #_sprite_0
	call cpct_drawSprite_asm
	jp drawTime_sec2
xx1x:
	call xxnx
	ld hl, #_sprite_1
	call cpct_drawSprite_asm
	jp drawTime_sec2
xx2x:
	call xxnx
	ld hl, #_sprite_2
	call cpct_drawSprite_asm
	jp drawTime_sec2
xx3x:
	call xxnx
	ld hl, #_sprite_3
	call cpct_drawSprite_asm
	jp drawTime_sec2
xx4x:
	call xxnx
	ld hl, #_sprite_4
	call cpct_drawSprite_asm
	jp drawTime_sec2
xx5x:
	call xxnx
	ld hl, #_sprite_5
	call cpct_drawSprite_asm
	jp drawTime_sec2

xxx0:
	call xxxn
	ld hl, #_sprite_0
	call cpct_drawSprite_asm
	jp drawTime_min1
xxx1:
	call xxxn
	ld hl, #_sprite_1
	call cpct_drawSprite_asm
	jp drawTime_min1
xxx2:
	call xxxn
	ld hl, #_sprite_2
	call cpct_drawSprite_asm
	jp drawTime_min1
xxx3:
	call xxxn
	ld hl, #_sprite_3
	call cpct_drawSprite_asm
	jp drawTime_min1
xxx4:
	call xxxn
	ld hl, #_sprite_4
	call cpct_drawSprite_asm
	jp drawTime_min1
xxx5:
	call xxxn
	ld hl, #_sprite_5
	call cpct_drawSprite_asm
	jp drawTime_min1
xxx6:
	call xxxn
	ld hl, #_sprite_6
	call cpct_drawSprite_asm
	jp drawTime_min1
xxx7:
	call xxxn
	ld hl, #_sprite_7
	call cpct_drawSprite_asm
	jp drawTime_min1
xxx8:
	call xxxn
	ld hl, #_sprite_8
	call cpct_drawSprite_asm
	jp drawTime_min1
xxx9:
	call xxxn
	ld hl, #_sprite_9
	call cpct_drawSprite_asm
	jp drawTime_min1

y0xxx:
	call nxxx
	ld hl, #_sprite_0
	call cpct_drawSprite_asm
	jp drawTime_min2
y1xxx:
	call nxxx
	ld hl, #_sprite_1
	call cpct_drawSprite_asm
	jp drawTime_min2
y2xxx:
	call nxxx
	ld hl, #_sprite_2
	call cpct_drawSprite_asm
	jp drawTime_min2
y3xxx:
	call nxxx
	ld hl, #_sprite_3
	call cpct_drawSprite_asm
	jp drawTime_min2
y4xxx:
	call nxxx
	ld hl, #_sprite_4
	call cpct_drawSprite_asm
	jp drawTime_min2
y5xxx:
	call nxxx
	ld hl, #_sprite_5
	call cpct_drawSprite_asm
	jp drawTime_min2
y6xxx:
	call nxxx
	ld hl, #_sprite_6
	call cpct_drawSprite_asm
	jp drawTime_min2
y7xxx:
	call nxxx
	ld hl, #_sprite_7
	call cpct_drawSprite_asm
	jp drawTime_min2
y8xxx:
	call nxxx
	ld hl, #_sprite_8
	call cpct_drawSprite_asm
	jp drawTime_min2
y9xxx:
	call nxxx
	ld hl, #_sprite_9
	call cpct_drawSprite_asm
	jp drawTime_min2

x0xx:
	call xnxx
	ld hl, #_sprite_0
	call cpct_drawSprite_asm
	ret
x1xx:
	call xnxx
	ld hl, #_sprite_1
	call cpct_drawSprite_asm
	ret
x2xx:
	call xnxx
	ld hl, #_sprite_2
	call cpct_drawSprite_asm
	ret
x3xx:
	call xnxx
	ld hl, #_sprite_3
	call cpct_drawSprite_asm
	ret
x4xx:
	call xnxx
	ld hl, #_sprite_4
	call cpct_drawSprite_asm
	ret
x5xx:
	call xnxx
	ld hl, #_sprite_5
	call cpct_drawSprite_asm
	ret
x6xx:
	call xnxx
	ld hl, #_sprite_6
	call cpct_drawSprite_asm
	ret
x7xx:
	call xnxx
	ld hl, #_sprite_7
	call cpct_drawSprite_asm
	ret
x8xx:
	call xnxx
	ld hl, #_sprite_8
	call cpct_drawSprite_asm
	ret
x9xx:
	call xnxx
	ld hl, #_sprite_9
	call cpct_drawSprite_asm
	ret

.globl _sprite_level
.globl _sprite_b0
.globl _sprite_b1
.globl _sprite_b2
.globl _sprite_b3
.globl _sprite_b4
.globl _sprite_b5
.globl _sprite_b6
.globl _sprite_b7
.globl _sprite_b8
.globl _sprite_b9
.globl _sprite_bsep

lvl1:
	ld hl, #_sprite_b1
	jp drawLevel_continue

lvl2:
	ld hl, #_sprite_b2
	jp drawLevel_continue

lvl3:
	ld hl, #_sprite_b3
	jp drawLevel_continue

lvl4:
	ld hl, #_sprite_b4
	jp drawLevel_continue

drawLevel::
	call fade
	ld bc, #0x5717
	ld de, (ptrVideo)
	call cpct_getScreenPtr_asm
	push hl
	ex de, hl
	ld bc, #0x1A1B
	ld hl, #_sprite_level
	call cpct_drawSprite_asm
	pop hl
	ld bc, #29
	add hl, bc
	ex de, hl
	ld a, (level)
	cp a, #1
	jp z, lvl1
	cp a, #2
	jp z, lvl2
	cp a, #3
	jp z, lvl3
	jp lvl4
drawLevel_continue:
	ld bc, #0x1A05
	call cpct_drawSprite_asm
	call cpct_waitVSYNC_asm
    jp switchBuffers

.globl _sprite_gameover
.globl _sprite_playagain

drawGameOver::
	call fade
	ld bc, #0x4B11
	ld de, (ptrVideo)
	call cpct_getScreenPtr_asm
	ex de, hl
	ld bc, #0x1A2E
	ld hl, #_sprite_gameover
	call cpct_drawSprite_asm
	ld bc, #0x711E
	ld de, (ptrVideo)
	call cpct_getScreenPtr_asm
	ex de, hl
	ld bc, #0x0C15
	ld hl, #_sprite_playagain
	call cpct_drawSprite_asm
	call cpct_waitVSYNC_asm
    jp switchBuffers

.globl _sprite_youwin

drawGameWin::
	call fade
	ld bc, #0x4412
	ld de, (ptrVideo)
	call cpct_getScreenPtr_asm
	ex de, hl
	ld bc, #0x1A2C
	ld hl, #_sprite_youwin
	call cpct_drawSprite_asm
	ld a, (sec)
drawTime_bsec1:
	sub a, #10
	jp m, bxx0x
	sub a, #10
	jp m, bxx1x
	sub a, #10
	jp m, bxx2x
	sub a, #10
	jp m, bxx3x
	sub a, #10
	jp m, bxx4x
	sub a, #10
	jp bxx5x
drawTime_bsec2:
	ld a, (sec_aux)
	inc a
	jp p, bxxx9
	inc a
	jp p, bxxx8
	inc a
	jp p, bxxx7
	inc a
	jp p, bxxx6
	inc a
	jp p, bxxx5
	inc a
	jp p, bxxx4
	inc a
	jp p, bxxx3
	inc a
	jp p, bxxx2
	inc a
	jp p, bxxx1
	jp bxxx0
drawTime_bmin1:
	ld bc, #0x6A27
	ld de, (ptrVideo)
	call cpct_getScreenPtr_asm
	ex de, hl
	ld bc, #0x1A02
	ld hl, #_sprite_bsep
	call cpct_drawSprite_asm
	ld (refresh_min), a
	ld a, (min)
	sub a, #10
	jp m, by0xxx
	sub a, #10
	jp m, by1xxx
	sub a, #10
	jp m, by2xxx
	sub a, #10
	jp m, by3xxx
	sub a, #10
	jp m, by4xxx
	sub a, #10
	sub a, #10
	jp m, by5xxx
	sub a, #10
	jp m, by6xxx
	sub a, #10
	jp m, by7xxx
	sub a, #10
	jp m, by8xxx
	sub a, #10
	jp by9xxx
drawTime_bmin2:
	ld a, (min_aux)
	cp a, #0
	jp p, bx9xx
	inc a
	jp p, bx9xx
	inc a
	jp p, bx8xx
	inc a
	jp p, bx7xx
	inc a
	jp p, bx6xx
	inc a
	jp p, bx5xx
	inc a
	jp p, bx4xx
	inc a
	jp p, bx3xx
	inc a
	jp p, bx2xx
	inc a
	jp p, bx1xx
	jp bx0xx
drawTime_bmin3:
	call cpct_waitVSYNC_asm
    jp switchBuffers
bnnnn1:
	ld de, (ptrVideo)
	call cpct_getScreenPtr_asm
	ex de, hl
	ld bc, #0x1A05
	ret
bxxnx:
	ld (sec_aux), a
	ld bc, #0x6A2A
	call bnnnn1
	ret
bxxxn:
	ld bc, #0x6A31
	call bnnnn1
	ret
bnxxx:
	ld (min_aux), a
	ld bc, #0x6A1A
	call bnnnn1
	ret
bxnxx:
	ld bc, #0x6A21
	call bnnnn1
	ret
bxx0x:
	call bxxnx
	ld hl, #_sprite_b0
	call cpct_drawSprite_asm
	jp drawTime_bsec2
bxx1x:
	call bxxnx
	ld hl, #_sprite_b1
	call cpct_drawSprite_asm
	jp drawTime_bsec2
bxx2x:
	call bxxnx
	ld hl, #_sprite_b2
	call cpct_drawSprite_asm
	jp drawTime_bsec2
bxx3x:
	call bxxnx
	ld hl, #_sprite_b3
	call cpct_drawSprite_asm
	jp drawTime_bsec2
bxx4x:
	call bxxnx
	ld hl, #_sprite_b4
	call cpct_drawSprite_asm
	jp drawTime_bsec2
bxx5x:
	call bxxnx
	ld hl, #_sprite_b5
	call cpct_drawSprite_asm
	jp drawTime_bsec2

bxxx0:
	call bxxxn
	ld hl, #_sprite_b0
	call cpct_drawSprite_asm
	jp drawTime_bmin1
bxxx1:
	call bxxxn
	ld hl, #_sprite_b1
	call cpct_drawSprite_asm
	jp drawTime_bmin1
bxxx2:
	call bxxxn
	ld hl, #_sprite_b2
	call cpct_drawSprite_asm
	jp drawTime_bmin1
bxxx3:
	call bxxxn
	ld hl, #_sprite_b3
	call cpct_drawSprite_asm
	jp drawTime_bmin1
bxxx4:
	call bxxxn
	ld hl, #_sprite_b4
	call cpct_drawSprite_asm
	jp drawTime_bmin1
bxxx5:
	call bxxxn
	ld hl, #_sprite_b5
	call cpct_drawSprite_asm
	jp drawTime_bmin1
bxxx6:
	call bxxxn
	ld hl, #_sprite_b6
	call cpct_drawSprite_asm
	jp drawTime_bmin1
bxxx7:
	call bxxxn
	ld hl, #_sprite_b7
	call cpct_drawSprite_asm
	jp drawTime_bmin1
bxxx8:
	call bxxxn
	ld hl, #_sprite_b8
	call cpct_drawSprite_asm
	jp drawTime_bmin1
bxxx9:
	call bxxxn
	ld hl, #_sprite_b9
	call cpct_drawSprite_asm
	jp drawTime_bmin1

by0xxx:
	call bnxxx
	ld hl, #_sprite_b0
	call cpct_drawSprite_asm
	jp drawTime_bmin2
by1xxx:
	call bnxxx
	ld hl, #_sprite_b1
	call cpct_drawSprite_asm
	jp drawTime_bmin2
by2xxx:
	call bnxxx
	ld hl, #_sprite_b2
	call cpct_drawSprite_asm
	jp drawTime_bmin2
by3xxx:
	call bnxxx
	ld hl, #_sprite_b3
	call cpct_drawSprite_asm
	jp drawTime_bmin2
by4xxx:
	call bnxxx
	ld hl, #_sprite_b4
	call cpct_drawSprite_asm
	jp drawTime_bmin2
by5xxx:
	call bnxxx
	ld hl, #_sprite_b5
	call cpct_drawSprite_asm
	jp drawTime_bmin2
by6xxx:
	call bnxxx
	ld hl, #_sprite_b6
	call cpct_drawSprite_asm
	jp drawTime_bmin2
by7xxx:
	call bnxxx
	ld hl, #_sprite_b7
	call cpct_drawSprite_asm
	jp drawTime_bmin2
by8xxx:
	call bnxxx
	ld hl, #_sprite_b8
	call cpct_drawSprite_asm
	jp drawTime_bmin2
by9xxx:
	call bnxxx
	ld hl, #_sprite_b9
	call cpct_drawSprite_asm
	jp drawTime_bmin2
bx0xx:
	call bxnxx
	ld hl, #_sprite_b0
	call cpct_drawSprite_asm
	jp drawTime_bmin3
bx1xx:
	call bxnxx
	ld hl, #_sprite_b1
	call cpct_drawSprite_asm
	jp drawTime_bmin3
bx2xx:
	call bxnxx
	ld hl, #_sprite_b2
	call cpct_drawSprite_asm
	jp drawTime_bmin3
bx3xx:
	call bxnxx
	ld hl, #_sprite_b3
	call cpct_drawSprite_asm
	jp drawTime_bmin3
bx4xx:
	call bxnxx
	ld hl, #_sprite_b4
	call cpct_drawSprite_asm
	jp drawTime_bmin3
bx5xx:
	call bxnxx
	ld hl, #_sprite_b5
	call cpct_drawSprite_asm
	jp drawTime_bmin3
bx6xx:
	call bxnxx
	ld hl, #_sprite_b6
	call cpct_drawSprite_asm
	jp drawTime_bmin3
bx7xx:
	call bxnxx
	ld hl, #_sprite_b7
	call cpct_drawSprite_asm
	jp drawTime_bmin3
bx8xx:
	call bxnxx
	ld hl, #_sprite_b8
	call cpct_drawSprite_asm
	jp drawTime_bmin3
bx9xx:
	call bxnxx
	ld hl, #_sprite_b9
	call cpct_drawSprite_asm
	jp drawTime_bmin3