;;-----------------------------LICENSE NOTICE-----------------------------------
;;  This file is part of Turtle Slam: An Amstrad CPC Game
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU General Public License for more details.
;;
;;  You should have received a copy of the GNU General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;------------------------------------------------------------------------------

.area _DATA
.area _CODE

.include "map.h.s"
.include "cpctelera.h.s"
.include "drawer.h.s"
.include "timer.h.s"
.include "video.h.s"
.include "hero.h.s"
.include "bullet.h.s"
.include "enemy.h.s"
.include "models/hero.h.s"
.include "models/room.h.s"
.include "keyboard/keyboard.h.s"

max_level = 5

level::
	.db #1

ptrRoom::
	.dw #0

getRoom::
	ld hl, (ptrRoom)
	ld c, (hl)
	inc hl
	ld b, (hl)
	push bc
	pop iy
	ret

bullet_turn:
	.db #0

game::
	call fade
	;; Restore level
	ld a, #1
	ld (level), a

	;; TODO 
	;; restore hero and enemys stats
	ld a, #3
	ld (health), a
	ld a, #5
	ld (pause_mele), a
	ld a, #70
	ld (pause_ranged), a

	ld a, #4
	ld ix, #hero
	ld h_hp(ix), a
	ld a, #0
	ld (hero_inv), a
	ld (hero_inv_counter), a

	;; Generate map
	call generateMap

	call drawLevel
	ld a, #100
	call wait

	;; Reset timer
	call resetTimer
	;; Draw room
	call changeRoom
	game_loop:
		;; Erase enemy
		call getRoom
		ld a, r_v(iy)
		cp a, #0
		call z, eraseEnemy
		call eraseBullet
		;; Hero
		ld ix, #hero
        call erase
		call controlBullet
		call cpct_scanKeyboard_f_asm
		ld ix, #hero
        call controlHero
		ld a, h_hp(ix)
		cp a, #0
		jp z, game_over
		call drawBullet
		ld a, (hero_inv_counter)
		bit 0, a
		jp nz, game_loop_enemy
		ld ix, #hero
		call draw
		;; Enemy
	game_loop_enemy:
		call getRoom
		call controlEnemy
	continue_game_loop:
		call drawEnemy
		ld a, (level)
		cp a, #max_level
		jp z, game_win
		call drawHp
		call drawTime
		call cpct_waitVSYNC_asm
        call switchBuffers
	jp game_loop

aux_wait:
	.db #0

wait:
	ld (aux_wait), a
	call cpct_scanKeyboard_f_asm
    ld hl, #Key_Return
    call cpct_isKeyPressed_asm
    ret nz
	call cpct_waitVSYNC_asm
	ld a, (aux_wait)
	dec a
	ld (aux_wait), a
	ret z
	jp wait

game_over:
	call fade
	call drawGameOver
	ld a, #0
	jp wait

game_win:
	call fade
	call drawGameWin
	ld a, #0
	call wait
	jp wait

increaseHealth::
	ld a, (health)
	add a, #2
	ld (health), a
	ret

increaseDif::
	ld a, (pause_ranged)
	sub a, #15
	ld (pause_ranged), a
	ld a, (pause_mele)
	sub a, #1
	ld (pause_mele), a
	ret

increaseHp:
	ld ix, #hero
	ld a, h_hp(ix)
	cp a, #4
	ret z
	inc a
	ld h_hp(ix), a
	ret

levelUp::
	pop hl
	call fade
	ld a, (level)
	inc a
	cp a, #max_level
	ld (level), a
	ret z
	call increaseHp
	ld a, (level)
	bit 0, a
	call nz, increaseHealth
	ld a, (level)
	bit 0, a
	call z, increaseDif
	call drawLevel
	ld a, #100
	call wait
	call generateMap
	jp changeRoom

setRoom::
	ld hl, #arrayRooms
	ld bc, #2
	findRoom:
		cp a, #0
		jp z, storeRoom
		dec a
		add hl, bc
	jp findRoom
	storeRoom:
		ld (ptrRoom), hl
	ret

roomUp::
	pop hl
	ld hl, (ptrRoom)
	dec hl
	dec hl
	dec hl
	dec hl
	dec hl
	dec hl
	ld (ptrRoom), hl
	ld bc, #0x26B4
	call setHeroPos
	jp changeRoom

roomDown::
	pop hl	
	ld hl, (ptrRoom)
	inc hl
	inc hl
	inc hl
	inc hl
	inc hl
	inc hl
	ld (ptrRoom), hl
	ld bc, #0x2610
	call setHeroPos
	jp changeRoom

roomRight::
	pop hl
	ld hl, (ptrRoom)
	inc hl
	inc hl
	ld (ptrRoom), hl
	ld bc, #0x0260
	call setHeroPos
	jp changeRoom

roomLeft::
	pop hl
	ld hl, (ptrRoom)
	dec hl
	dec hl
	ld (ptrRoom), hl
	ld bc, #0x4A60
	call setHeroPos
	jp changeRoom

changeRoom:
	ld a, #3
	ld (refresh_sec), a
	ld (refresh_min), a
	call disableEnemy
	call disableBullet
	call fade
	;; New room buffer 1
	call getRoom
	call drawRoom
	call getRoom
	ld a, r_v(iy)
	cp a, #0
	call z, loadEnemy
	call cpct_waitVSYNC_asm
	call unfade
	call switchBuffers
	;; New room buffer 2
	call getRoom
	jp drawRoom

openDoors::
	call getRoom
	ld a, r_v(iy)
	cp a, #1
	ret z
	ld a, #1
	ld r_v(iy), a
	call drawOpenDoors
	call cpct_waitVSYNC_asm
	call switchBuffers
	call getRoom
	call drawOpenDoors
	call cpct_waitVSYNC_asm
	jp switchBuffers