;;-----------------------------LICENSE NOTICE-----------------------------------
;;  This file is part of Turtle Slam: An Amstrad CPC Game
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU General Public License for more details.
;;
;;  You should have received a copy of the GNU General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;------------------------------------------------------------------------------

.area _DATA
.area _CODE

.include "keyboard/keyboard.h.s"
.include "models/hero.h.s"
.include "models/room.h.s"
.include "game.h.s"
.include "cpctelera.h.s"
.include "drawer.h.s"
.include "video.h.s"
.include "bullet.h.s"
.include "models/bullet.h.s"

;; Hero sprites
.globl _sprite_hero_u0
.globl _sprite_hero_u1
.globl _sprite_hero_d0
.globl _sprite_hero_d1
.globl _sprite_hero_r0
.globl _sprite_hero_r1
.globl _sprite_hero_l0
.globl _sprite_hero_l1

hero::
	DefineDefaultHero _sprite_hero_u0

hero_inv::
	.db #0

hero_inv_counter::
	.db #0

hero_shoot::
	.db #0

hero_cadency::
	.db #30

setHeroPos::
	ld ix, #hero
	ld e_x(ix), b
	ld h_ox(ix), b
	ld e_y(ix), c
	ld h_oy(ix), c
	ret

;; ============================================================================
;; Control hero
;; Destroyed register values: A, BC, D, HL
;; ============================================================================
controlHero::
	ld a, (hero_shoot)
	cp a, #0
	jp z, controlHeroInv
	dec a
	ld (hero_shoot), a
controlHeroInv:
	ld a, (hero_inv)
	cp a, #0
	jp z, controlHeroMove
	ld a, (hero_inv_counter)
	dec a
	ld (hero_inv_counter), a
	jp nz, controlHeroMove
	ld (hero_inv), a
controlHeroMove:
	;; Move up
	;; Keyboard
	ld hl, #Key_W
	call cpct_isKeyPressed_asm
	call nz, moveHeroUp
	;; Joystick 0
	;; ld hl, #Joy0_Up
	;; call cpct_isKeyPressed_asm
	;; call nz, moveHeroUp
	;; Move down
	;; Keyboard
	ld hl, #Key_S
	call cpct_isKeyPressed_asm
	call nz, moveHeroDown
	;; Joystick 0
	;; ld hl, #Joy0_Down
	;; call cpct_isKeyPressed_asm
	;; call nz, moveHeroDown
	;; Move right
	;; Keyboard
	ld hl, #Key_D
	call cpct_isKeyPressed_asm
	call nz, moveHeroRight
	;; Joystick 0
	;; ld hl, #Joy0_Right
	;; call cpct_isKeyPressed_asm
	;; call nz, moveHeroRight
	;; Move left
	;; Keyboard
	ld hl, #Key_A
	call cpct_isKeyPressed_asm
	call nz, moveHeroLeft
	;; Joystick 0
	;; ld hl, #Joy0_Left
	;; call cpct_isKeyPressed_asm
	;; call nz, moveHeroLeft	
	ld a, (hero_shoot)
	cp a, #0
	ret nz
	;; Shoot up
	;; Keyboard
	ld hl, #Key_I
	call cpct_isKeyPressed_asm
	jp nz, shootUp
	;; Joystick 0
	ld hl, #Joy0_Up
	call cpct_isKeyPressed_asm
	jp nz, shootUp
	;; Shoot down
	;; Keyboard
	ld hl, #Key_K
	call cpct_isKeyPressed_asm
	jp nz, shootDown
	;; Joystick 0
	ld hl, #Joy0_Down
	call cpct_isKeyPressed_asm
	jp nz, shootDown
	;; Shoot right
	;; Keyboard
	ld hl, #Key_L
	call cpct_isKeyPressed_asm
	jp nz, shootRight
	;; Joystick 0
	ld hl, #Joy0_Right
	call cpct_isKeyPressed_asm
	jp nz, shootRight
	;; Shoot left
	;; Keyboard
	ld hl, #Key_J
	call cpct_isKeyPressed_asm
	jp nz, shootLeft
	;; Joystick 0
	ld hl, #Joy0_Left
	call cpct_isKeyPressed_asm
	jp nz, shootLeft
	ret

status:
	.db #0

collision:
	ld a, #0
	ld (status), a
	ld de, #hero_obstacle_collision
	ld (collision_function), de
	ld a, b ;; X1
	add a, e_w(ix)
	ld d, a ;; X2
	ld a, c ;; Y1
	add a, e_h(ix)
	ld e, a ;; Y2

	ld l, r_pt_l(iy)
	ld h, r_pt_h(iy)
	push hl
	pop iy
	loop_vc:
		ld a, 0(iy)
		cp a, #255
		jp z, set_spike
	horizontal1:
		ld a, e_x(iy)
		sub a, b
		jp z, vertical1
		jp m, horizontal11
	horizontal2:
		ld a, e_x(iy)
		sub a, d
		jp m, horizontal22
		jp next_vc
	vertical1:
		ld a, e_y(iy)
		sub a, c
		jp m, vertical11
	vertical2:
		ld a, e_y(iy)
		sub a, e
		jp m, vertical22
	next_vc:
		push bc
		ld bc, #e_size
		add iy, bc
		pop bc
		jp loop_vc
	horizontal11:
		add a, e_w(iy)
		dec a
		jp m, horizontal2
		jp vertical1
	horizontal22:
		add a, e_w(iy)
		dec a
		jp m, next_vc
		jp vertical1
	vertical11:
		add a, e_h(iy)
		dec a
		jp m, vertical2
		jp collided
	vertical22:
		add a, e_h(iy)
		dec a
		jp m, next_vc
	collided:
		collision_function = . + 1
		jp hero_obstacle_collision
	set_spike:
		ld a, (status)
		cp a, #0
		ret nz
		inc a
		ld (status), a
		inc iy
		push de
		ld de, #hero_spike_collision
		ld (collision_function), de
		pop de
		jp loop_vc


hero_obstacle_collision:
	pop hl
	pop hl
	ret

hero_spike_collision:
	pop hl
	pop hl
	pop hl
	ld a, (hero_inv)
	cp a, #1
	ret z
	push af
	push ix
	push iy
	ld l, #02
	ld h, #15
	ld e, #58
	ld d, #01
	ld bc, #0000
	ld a, #0b00000100
	call cpct_akp_SFXPlay_asm
	pop iy
	pop ix
	pop af
	ld a, h_hp(ix)
	dec a
	ld h_hp(ix), a
	ret z
	ld a, #1
	ld (hero_inv), a
	ld a, #30
	ld (hero_inv_counter), a
	ret

verticalCollision:
	push af
	call getRoom
	pop af
	ld b, e_x(ix) 
	ld c, a
	jp collision

horizontalCollision:
	push af
	call getRoom
	pop af
	ld b, a
	ld c, e_y(ix) 
	jp collision

;; ============================================================================
;; Move hero
;; Destroyed register values: A, B
;; ============================================================================
;; Up
moveHeroUp::
	;; Animation
	call updateSpriteStatus
		ld a, h_ss(ix)
		cp a, #0
		jp z, setSpriteUp
	resetSpriteUp:
		ld hl, #_sprite_hero_u0
		jp spriteUp
	setSpriteUp:
		ld hl, #_sprite_hero_u1
	spriteUp:
		ld h_pSpr_l(ix), l
		ld h_pSpr_h(ix), h
	;; Motion
	ld a, e_y(ix)
	cp a, #rd_upl
	jp z, upper_limit
	sub a, #4
	push af
	call verticalCollision
	pop af
	ld e_y(ix), a
	ret
	upper_limit:
		call getRoom
		ld b, rd_up(iy)
		ld a, r_v(iy)
		cp a, #0
		ret z
		ld a, e_x(ix)
		cp a, #37
		jp z, upper_check
		cp a, #38
		jp z, upper_check
		cp a, #39
		jp z, upper_check
		ret
	upper_check:
		ld a, b
		cp a, #1
		jp z, roomUp
		cp a, #2
		jp z, levelUp
		ret
;; Down
moveHeroDown::
	call updateSpriteStatus
		ld a, h_ss(ix)
		cp a, #0
		jp z, setSpriteDown
	resetSpriteDown:
		ld hl, #_sprite_hero_d0
		jp spriteDown
	setSpriteDown:
		ld hl, #_sprite_hero_d1
	spriteDown:
		ld h_pSpr_l(ix), l
		ld h_pSpr_h(ix), h
	ld a, e_y(ix)
	ld b, a
	add a, e_h(ix)
	cp a, #rd_lol
	jp z, lower_limit
	ld a, b
	add a, #4
	push af
	call verticalCollision
	pop af
	ld e_y(ix), a
	ret
	lower_limit:
		call getRoom
		ld b, rd_lo(iy)
		ld a, r_v(iy)
		cp a, #0
		ret z
		ld a, e_x(ix)
		cp a, #37
		jp z, lower_check
		cp a, #38
		jp z, lower_check
		cp a, #39
		jp z, lower_check
		ret
	lower_check:
		ld a, b
		cp a, #1
		jp z, roomDown
		cp a, #2
		jp z, levelUp
		ret
;; Right
moveHeroRight::
	call updateSpriteStatus
		ld a, h_ss(ix)
		cp a, #0
		jp z, setSpriteRight
	resetSpriteRight:
		ld hl, #_sprite_hero_r0
		jp spriteRight
	setSpriteRight:
		ld hl, #_sprite_hero_r1
	spriteRight:
		ld h_pSpr_l(ix), l
		ld h_pSpr_h(ix), h
	ld a, e_x(ix)
	ld b, a
	add a, e_w(ix)
	cp a, #rd_ril
	jp z, right_limit
	ld a, b
	inc a
	push af
	call horizontalCollision
	pop af
	ld e_x(ix), b
	ret
	right_limit:
		call getRoom
		ld b, rd_ri(iy)
		ld a, r_v(iy)
		cp a, #0
		ret z
		ld a, e_y(ix)
		cp a, #96
		jp z, right_check
		cp a, #100
		jp z, right_check
		ret
	right_check:
		ld a, b
		cp a, #1
		jp z, roomRight
		cp a, #2
		jp z, levelUp
		ret
;; Left
moveHeroLeft::
	call updateSpriteStatus
		ld a, h_ss(ix)
		cp a, #0
		jp z, setSpriteLeft
	resetSpriteLeft:
		ld hl, #_sprite_hero_l0
		jp spriteLeft
	setSpriteLeft:
		ld hl, #_sprite_hero_l1
	spriteLeft:
		ld h_pSpr_l(ix), l
		ld h_pSpr_h(ix), h
	ld a, e_x(ix)
	cp a, #rd_lel
	jp z, left_limit
	dec a
	push af
	call horizontalCollision
	pop af
	ld e_x(ix), a
	ret
	left_limit:
		call getRoom
		ld b, rd_le(iy)
		ld a, r_v(iy)
		cp a, #0
		ret z
		ld a, e_y(ix)
		cp a, #96
		jp z, left_check
		cp a, #100
		jp z, left_check
		ret
	left_check:
		ld a, b
		cp a, #1
		jp z, roomLeft
		cp a, #2
		jp z, levelUp
		ret

;; ============================================================================
;; Update sprite status
;; Destroyed register values: A
;; ============================================================================
updateSpriteStatus:
	ld a, h_ssc(ix)
	dec a
	jp z, switchSatus
	ld h_ssc(ix), a
	ret
	switchSatus:
		ld a, #h_ss_freq
		ld h_ssc(ix), a
		ld a, h_ss(ix)
		xor a, #1
		ld h_ss(ix), a
	ret

;; ============================================================================
;; Shoot
;; ============================================================================
shootUp:
	call findBullet
	ld a, e_x(ix)
	add a, #2
	ld b_x(iy), a
	ld a, e_y(ix)
	add a, #4
	ld b_y(iy), a
	ld a, #2
	ld b_di(iy), a
	jp enableBullet
shootDown:
	call findBullet
	ld a, e_x(ix)
	add a, #1
	ld b_x(iy), a
	ld a, e_y(ix)
	add a, #8
	ld b_y(iy), a
	ld a, #4
	ld b_di(iy), a
	jp enableBullet
shootRight:
	call findBullet
	ld a, e_x(ix)
	add a, #2
	ld b_x(iy), a
	ld a, e_y(ix)
	add a, #4
	ld b_y(iy), a
	ld a, #3
	ld b_di(iy), a
	jp enableBullet
shootLeft:
	call findBullet
	ld a, e_x(ix)
	add a, #1
	ld b_x(iy), a
	ld a, e_y(ix)
	add a, #4
	ld b_y(iy), a
	ld a, #5
	ld b_di(iy), a
	jp enableBullet

enableBullet:
	ld a, #1
	ld b_si(iy), a
	ld a, #2
	ld b_st(iy), a
	ld a, b_x(iy)
	ld b_ox(iy), a
	ld a, b_y(iy)
	ld b_oy(iy), a
	ret

findBullet:
	ld a, (hero_cadency)
	ld (hero_shoot), a
	ld iy, #bullets
	loop_findBullet:
		ld a, 0(iy)
		cp a, #255
		jp z, not_findBullet
		ld a, b_st(iy)
		cp a, #0
		ret z
	next_findBullet:
		ld bc, #b_size
		add iy, bc
	jp loop_findBullet
	not_findBullet:
		pop hl
		ret