;;-----------------------------LICENSE NOTICE-----------------------------------
;;  This file is part of Turtle Slam: An Amstrad CPC Game
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU General Public License for more details.
;;
;;  You should have received a copy of the GNU General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;------------------------------------------------------------------------------

.area _DATA
.area _CODE

.include "tilemaps/myfinalset_pack.h.s"
.include "tilemaps/tilemap.h.s"
.include "keyboard/keyboard.h.s"
.include "cpctelera.h.s"
.include "drawer.h.s"
.include "timer.h.s"
.include "video.h.s"
.include "game.h.s"
.include "hero.h.s"
.include "menu.h.s"
.globl _song_ingame
.globl _song_fsx

;; ============================================================================
;; Main function
;; ============================================================================
_main::
    ;; Change stack pointer
    ld sp, #0x8000
    ;; Initialize
    call initialize
    main_loop:
        call menu
        call game
    jp main_loop

;; ============================================================================
;; Interrupt Service Routine (ISR)
;; Saved register values: AF, BC, DE, HL, IX
;; Not saved register values: AF', BC', DE', HL'
;; ============================================================================
variable: 
    .db #12
isr:
    call tick
    ld a, (variable)
	dec a
	ld (variable), a
    ret nz
    ld a, #12
    ld (variable), a
	jp cpct_akp_musicPlay_asm

;; ============================================================================
;; Initialization
;; Destroyed register values: AF, BC, DE, HL
;; ============================================================================
initialize:
    ;; Set music

    ld de, #_song_ingame
    call cpct_akp_musicInit_asm
    ld de, #_song_fsx
    call cpct_akp_SFXInit_asm
    ;; Disable firmware
    call cpct_disableFirmware_asm
    ;; Set video mode
    ld c, #video_mode
    call cpct_setVideoMode_asm
    ;; Set palette
    ld hl, #_palette
    ld de, #16
    call cpct_setPalette_asm
    ;; Set border color
    ld l, #16
    ld h, #border_color
    call cpct_setPALColour_asm
    ;; Initialize second buffer
    call initializeBuffer
    ;; Set ISR
    ld hl, #isr
    jp cpct_setInterruptHandler_asm