;;-----------------------------LICENSE NOTICE-----------------------------------
;;  This file is part of Turtle Slam: An Amstrad CPC Game
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU General Public License for more details.
;;
;;  You should have received a copy of the GNU General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;------------------------------------------------------------------------------

.area _DATA
.area _CODE

.include "random_rooms/random_room.h.s"
.include "random_maps/random_map.h.s"
.include "models/map.h.s"
.include "cpctelera.h.s"
.include "timer.h.s"
.include "game.h.s"
.include "hero.h.s"

map::
	DefineDefaultMap

arrayRooms::
	.dw #room0
	.dw #room1
	.dw #room2
	.dw #room3
	.dw #room4
	.dw #room5
	.dw #room6
	.dw #room7
	.dw #room8

.macro MapsRepetion
	.rept 32
		.db #0
	.endm
	.db #255
.endm

map_repetition:
	MapsRepetion

aux_room:
	.db #0

obtainRandomRoom:
	call getSeed
	call cpct_getRandom_lcg_u8_asm
	and a, #0b00011110
	ld (aux_room), a
	ld b, #0
	ld c, a
	ld hl, #map_repetition
	add hl, bc
	ld a, (hl)
	cp a, #0
	jp nz, obtainRandomRoom
	ld (hl), #1
	ld a, (aux_room)
	ret

generateMap::
	ld hl, #map_repetition
	generateMap_loop1:
		ld a, (hl)
		cp a, #255
		jp z, generateMap_continue1
		ld a, #0
		ld (hl), a
		inc hl
		jp generateMap_loop1
	generateMap_continue1:
	;; Set random seeder
	call getSeed
	call cpct_setSeed_lcg_u8_asm
	;; Random map generation
	call getSeed
	call cpct_getRandom_lcg_u8_asm
	and a, #0b00001110
	ld b, #0
	ld c, a
	ld ix, #arrayRandomMaps
	add ix, bc
	ld a, 0(ix)
	.dw #0x6FFD ;; ld iyl, a
	ld a, 1(ix)
	.dw #0x67FD ;; ld iyh, a
	;; Initial room
	ld a, 0(iy)
	inc iy
	call setRoom
	;; Intial hero position
	ld b, 0(iy)
	inc iy
	ld c, 0(iy)
	inc iy
	call setHeroPos
	;; Random rooms generation
	ld ix, #map
	generation_loop:
		;; Check loop end
		ld a, 0(ix)
		cp a, #255
		ret z
		;; Upper door
		ld a, rd_up(iy)
		ld rd_up(ix), a
		;; Lower door
		ld a, rd_lo(iy)
		ld rd_lo(ix), a
		;; Right door
		ld a, rd_ri(iy)
		ld rd_ri(ix), a
		;; Left door
		ld a, rd_le(iy)
		ld rd_le(ix), a
		;; PowerUp
		ld a, rd_p(iy)
		ld rd_p(ix), a
		;; Visited
		ld a, #0
		ld r_v(ix), a
		;; Room type	
		push ix
		ld ix, #arrayRandomRooms
		call obtainRandomRoom
		ld b, #0
		ld c, a
		add ix, bc
		ld c, 0(ix)
		ld b, 1(ix)
		pop ix
		ld r_pt_l(ix), c
		ld r_pt_h(ix), b
		;; Iteration
		ld bc, #r_size
		add ix, bc
		ld bc, #rd_size
		add iy, bc
	jp generation_loop